﻿package RedCorona.Components {
	import flash.display.DisplayObject;
	import flash.display.Sprite;
	import flash.events.MouseEvent;
	import flash.events.ProgressEvent;
	import RedCorona.Components.Events.DragEvent;
	/**
	 * ...
	 * @author Red Corona
	 */
	public class ScrollBar extends FocusSprite {
		private var topButton:SpriteButton, bottomButton:SpriteButton, tracker:SpriteButton;
		private var size:int, intendedheight:int, trackerSize:int, min:int, max:int, maxvalue:int;
		private var value:Number, dragStartValue:Number;
		
		public function get PageSize():int { return intendedheight; }
		
		public static const SCROLL_EVENT:String = 'scrollbarScroll';
		
		public function ScrollBar(h:int, min:int = 0, max:int = 400, value:int = 0, size:int = 12) {
			this.size = size;
			this.min = min; this.max = max; this.value = value;
			
			topButton = new SpriteButton();
			doButton(topButton, [ { x:0, y:-2 }, { x: -2, y: 2 }, { x:2, y: 2 }] );
			topButton.x = size / 2; topButton.y = 2 + (size / 2);
			topButton.tabEnabled = false;
			addChild(topButton);
			
			bottomButton = new SpriteButton();
			doButton(bottomButton, [ { x:0, y:2 }, { x: -2, y: -2 }, { x:2, y: -2 }] );
			bottomButton.x = size / 2; bottomButton.y = size / 2;
			bottomButton.tabEnabled = false;
			addChild(bottomButton);
			
			tracker = new SpriteButton();
			tracker.x = size / 2;
			addChild(tracker);
			tracker.addEventListener(DragEvent.ID, trackerDrag);
			tracker.addEventListener(MouseEvent.MOUSE_DOWN, trackerMouseDown);
			
			height = h;
		}
		
		public override function set height(value:Number) : void {
			intendedheight = value;
			bottomButton.y = (value - 2) - (size / 2);
			
			// track
			graphics.clear();
			graphics.beginFill(Colour.gamma(Settings.ButtonColour, 0.8), 0.5);
			graphics.drawRect(0, size / 2, size, value - size);
			graphics.endFill();
			
			// tracker
			fixTracker();
		}
		
		private function fixTracker() : void{
			trackerSize = (intendedheight - 2*size) * (intendedheight / max);
			if (intendedheight > max) {
				tracker.visible = false;
				maxvalue = value = 0;
			}
			else with (tracker.graphics) {
				tracker.visible = true;
				maxvalue = max - intendedheight;
				if (value > maxvalue) value = maxvalue;
				tracker.y = (size) + ((intendedheight - 2*size) * (value / max));
				clear();
				lineStyle(0, 0);
				beginFill(Settings.ButtonColour);
				if (trackerSize < 4) trackerSize = 4;
				drawRoundRect( -size / 2, 0, size, trackerSize, 4);
				endFill();				
			}
		}
		
		private function doButton(b:Sprite, points:Array) : void {	
			b.tabEnabled = false;
			b.addEventListener(MouseEvent.CLICK, buttonClick);
			var i:int;
			with (b.graphics) {
				clear();
				lineStyle(0, 0);
				beginFill(Settings.ButtonColour);
				drawCircle(0, 0, size / 2);
				endFill();
				beginFill(0);
				moveTo(points[0].x, points[0].y);
				for ( i = 1; i < points.length; i++)
					lineTo(points[i].x, points[i].y);
				endFill();
			}
		}
		
		private function buttonClick(e:MouseEvent) : void {
			Value = value + (intendedheight / 10) * ((e.target == topButton) ? -1 : 1);
		}
		
		public function get Value() : Number { return value; }
		public function set Value(newv:Number) : void {
			var old:Number = value;
			if (newv < 0) value = 0;
			else if (newv > maxvalue) value = maxvalue;
			else value = newv;
			
			tracker.y = (size) + ((height - 2 * size) * (value / max));
			
			//trace("Scroll, was " + old + " now " + value + " maxvalue "+maxvalue + " max "+max+" tracker at " + tracker.y);
			
			dispatchEvent(new ProgressEvent(SCROLL_EVENT, false, false, value - old, value));
		}
		
		public function get Max() : Number { return max; }
		public function set Max(v:Number) : void {
			max = v;
			fixTracker();
			//trace("New max: " + v + " maxvalue "+maxvalue+" max "+max+" height "+intendedheight);
			Value = value; // in case the limits changed
		}
		
		private function trackerMouseDown(e:MouseEvent) : void { dragStartValue = value; }		
		private function trackerDrag(e:DragEvent) : void {
			var dv:Number = e.location.y - e.start.y;
			//trace("Drag, dv " + dv + " start "+dragStartValue);
			Value = dragStartValue + dv * max / (height - 2 * size);
		}
	}
	
}