﻿package RedCorona.Components {
	import flash.display.DisplayObject;
	import flash.display.Sprite;
	import flash.events.KeyboardEvent;
	import flash.events.ProgressEvent;
	import flash.ui.Keyboard;
	
	/**
	 * ...
	 * @author Red Corona
	 */
	public class ScrollableRegion extends FocusSprite {
		private var vscroll:ScrollBar;
		private var clientArea:Sprite, scrolledPanel:Sprite, background:EdgedRectangle;
		private var w:int, h:int;
		private var distanceFromBottom:Number = 0;
		
		public var scrollFromBottom:Boolean;
		
		public function get Scrollbar() : ScrollBar { return vscroll; }
		
		public function ScrollableRegion(w:Number, h:Number, backCol:int = -1) {			
			clientArea = new Sprite(); scrolledPanel = new Sprite();
			vscroll = new ScrollBar(h);
			vscroll.addEventListener(ScrollBar.SCROLL_EVENT, scrolled);
			
			if (backCol == -1) backCol = Settings.ControlBackground;
			if (backCol != -2) {
				background = new EdgedRectangle(w, h, backCol, 0, 0, false, -2);
				super.addChild(background); 
			} else FocusFilter = null;
			
			// Must use super.addChild because ours is overridden to put controls inside the scrolling part
			super.addChild(clientArea); super.addChild(scrolledPanel); super.addChild(vscroll);
			
			scrolledPanel.mask = clientArea;
			scrolledPanel.graphics.beginFill(0, 0);
			scrolledPanel.graphics.drawCircle(1, 1, 1);
			scrolledPanel.graphics.endFill();
			
			addEventListener(KeyboardEvent.KEY_DOWN, keyDownHandler);
			
			this.w = w; this.h = h;
			redoSize();
		}
		
		public override function get numChildren() : int {
			return scrolledPanel.numChildren;
		}
		
		public override function getChildAt(i:int) : DisplayObject { return scrolledPanel.getChildAt(i); }
		
		public override function addChild(target:DisplayObject) : DisplayObject {
			target = scrolledPanel.addChild(target);
			redoSize();
			return target;
		}
		
		public override function removeChild(target:DisplayObject) : DisplayObject {
			target = scrolledPanel.removeChild(target);
			redoSize();
			return target;
		}
		
		public override function removeChildAt(i:int) : DisplayObject {
			var target:DisplayObject = scrolledPanel.removeChildAt(i);
			redoSize();
			return target;
		}
		
		public function appendChildAtBottom(target:DisplayObject) : DisplayObject {
			target.y = scrolledPanel.height;
			return addChild(target);
		}
		
		public override function set height(value:Number) : void { super.height = h = value; redoSize(); }
		public override function set width(value:Number) : void { super.width = w = value; redoSize(); }
		
		private function keyDownHandler(e:KeyboardEvent) : void {
			switch(e.keyCode){
				case Keyboard.DOWN: vscroll.Value += vscroll.PageSize * 0.1; break;
				case Keyboard.UP: vscroll.Value -= vscroll.PageSize * 0.1; break;
				case Keyboard.PAGE_DOWN: vscroll.Value += vscroll.PageSize * 0.8; break;
				case Keyboard.PAGE_UP: vscroll.Value -= vscroll.PageSize * 0.8; break;
			}
		}
		
		public function redoSize() : void {
			var w:Number = this.w;
			//trace("Scrollable region resized to [" + w + "," + h + "], scrolling " + scrolledPanel.y + " - " + scrolledPanel.height);
			if(background) background.redraw(w, h);
			vscroll.x = w - 10;
			vscroll.visible = h < scrolledPanel.height;
			vscroll.Max = scrolledPanel.height;
			if (vscroll.visible) {
				w -= 10;
				if (scrollFromBottom) {
					//trace('Scrolling to ' + distanceFromBottom + " from bottom: " + ( (vscroll.Max - vscroll.height) - distanceFromBottom ) + " (Max "+vscroll.Max+" height "+vscroll.height + ")");
					vscroll.Value = (vscroll.Max - vscroll.height) - distanceFromBottom;
				}
			} else vscroll.Value = 0;
			
			clientArea.graphics.clear();
			clientArea.graphics.beginFill(0, 0);
			clientArea.graphics.drawRect(-2, 0, w + 2, h);
			clientArea.graphics.endFill();			
		}
		
		private function scrolled(e:ProgressEvent) : void {
			if (e.bytesLoaded == 0) return; // no change
			scrolledPanel.y = -vscroll.Value;
			distanceFromBottom = (vscroll.Max - vscroll.height) - vscroll.Value;
		}
	}
	
}