﻿package RedCorona.Components {
	import flash.display.DisplayObject;
	import flash.display.Sprite;
	import flash.events.*;
	import flash.filters.BevelFilter;
	import flash.filters.ColorMatrixFilter;
	import flash.filters.DropShadowFilter;
	import flash.filters.GlowFilter;
	import flash.geom.Point;
	import RedCorona.Components.Events.DragEvent;
	
	/**
	 * ...
	 * @author Red Corona
	 */
	public class SpriteButton extends FocusSprite {
		private var over:Boolean, down:Boolean, keyDown:Boolean, enabled:Boolean = true, listensForEscape:Boolean;
		private var glowcol:int;
		private var downLocation:Point, lastpoint:Point;
		
		public function get Enabled() : Boolean { return enabled; }
		public function set Enabled(v:Boolean) : void {
			if (v == enabled) return;
			enabled = v;
			mouseEnabled = v;
			tabEnabled = v;
			adjustFilters();
		}
		
		public function get ListensForEscape() : Boolean { return listensForEscape; }
		public function set ListensForEscape(v: Boolean) : void { listensForEscape = v; }
		
		public function SpriteButton(glowcol:int = -1) {
			if (glowcol < 0) glowcol = Settings.GlowColour;
			this.glowcol = glowcol;
			
			addEventListener(MouseEvent.MOUSE_OVER, mouseOverOutHandler);
			addEventListener(MouseEvent.MOUSE_OUT, mouseOverOutHandler);
			
			addEventListener(MouseEvent.MOUSE_DOWN, mouseDownHandler);
			
			addEventListener(KeyboardEvent.KEY_DOWN, keyDownHandler);
			addEventListener(KeyboardEvent.KEY_UP, keyUpHandler);
			
			adjustFilters();
			
			tabEnabled = true; // buttonMode = true;			
		}
		
		public static function wrap(wrapped:DisplayObject) : SpriteButton {
			var button:SpriteButton = new SpriteButton();
			button.addChild(wrapped);
			return button;
		}
		
		protected override function adjustFilters() : void {
			super.adjustFilters();
			var filters:Array = this.filters;
			filters.push( new BevelFilter( (down || keyDown) ? 1 : 2) );
			if (!enabled) { 
				filters.push(Settings.InactiveFilter);
			} else {
				if (over) filters.push( new GlowFilter(glowcol) );
			}
			this.filters = filters;
		}
		
		private function mouseOverOutHandler(e:MouseEvent) : void {
			over = e.type == MouseEvent.MOUSE_OVER;
			adjustFilters();
		}
		
		private function mouseDownHandler(e:MouseEvent) : void {
			down = true;
			downLocation = lastpoint = new Point(e.stageX, e.stageY);
			stage.addEventListener(MouseEvent.MOUSE_UP, mouseUpHandler);			
			stage.addEventListener(MouseEvent.MOUSE_MOVE, mouseMoveHandler);			
			adjustFilters();
		}
		
		private function mouseUpHandler(e:MouseEvent) : void {
			down = false;
			stage.removeEventListener(MouseEvent.MOUSE_UP, mouseUpHandler);			
			stage.removeEventListener(MouseEvent.MOUSE_MOVE, mouseMoveHandler);			
			adjustFilters();
		}
		
		private function mouseMoveHandler(e:MouseEvent) : void {
			if (down) dispatchEvent(new DragEvent(new Point(e.stageX, e.stageY), downLocation, lastpoint ));
			lastpoint = new Point(e.stageX, e.stageY);
		}
		
		private function keyDownHandler(e:KeyboardEvent) : void {
			if (!enabled) return;
			if ((e.keyCode == 13) || (e.keyCode == 32) || (e.keyCode == 27 && listensForEscape) ) {
				keyDown = true;
				adjustFilters();
			}
		}
		private function keyUpHandler(e:KeyboardEvent) : void {
			if (!(keyDown && enabled)) return;
			if ((e.keyCode == 13) || (e.keyCode == 32) || (e.keyCode == 27 && listensForEscape) ) {
				keyDown = false;
				dispatchEvent(new MouseEvent(MouseEvent.CLICK));
				adjustFilters();
			}
		}
	}
	
	
}