package RedCorona.Tests {
	import flash.utils.getQualifiedClassName;
	
	/**
	* ...
	* @author 
	*/
	public class TestCase {
		public static const DEFAULT_TOLERANCE:Number = 1e-12;
		
		public var failed:Boolean;
		public var ignored:Boolean;
		public var name:String;
		public var time:int;
		public var hostClass:Class;
		
		public var status:String = 'OK';
		
		public function fail(message:String) : void {
			status = message;
			failed = true;
			trace(name + "Failed: " + message);
		}

		private static var currentTestCase:TestCase;
	
		public static function startTest(name:String) : TestCase {
			currentTestCase = new TestCase();
			currentTestCase.name = name;
			return currentTestCase;
		}
		
		public static function get current() : TestCase { return currentTestCase; }
		
		public static function isBaseType(obj:*) : Boolean {
			return obj is int || obj is Number || obj is Boolean || obj is String;
		}
		
		public function assertEqual(expected:*, actual:*, message:String = '') : void {
			if (expected == actual) return;
			
			if(!isBaseType(expected) ) {
				//trace('object or array check on '+expected);
				if(actual is Object || actual is Array)
					assertArrayEqual(expected, actual, message);
				else 
					fail("Expected " + message+" vector type but found " + actual);
			}
			else {
				if(expected != actual)
					fail("Expected "+message+": " + expected + " but was: " + actual);		
			}
		}
		
		public function assertAlmostEqual(expected:Number, actual:Number, message:String = '', tolerance:Number = DEFAULT_TOLERANCE) : void {
			var delta:Number = actual - expected;
			if (delta < -tolerance || delta > tolerance)
				fail("Expected "+message+": " + expected + " ±"+tolerance+" but was: " + actual);		
		}
		
		public function assertArrayEqual(expected:*, actual:*, message:String = '') : void {
			var expectedList:Vector.<Object> = new Vector.<Object>(),
				actualList:Vector.<Object> = new Vector.<Object>();
			
			for (var it:* in expected) expectedList.push( { key: it, value : expected[it] } );
			for (it in actual) actualList.push( { key: it, value : actual[it] } );
			
			if (actualList.length > 0) {
				if ( actualList.length == expectedList.length) {
					for (var i:int = 0; i < expectedList.length; i++) {
						var a:* = actualList[i].value, e:* = expectedList[i].value;
						if (a is Number && e is Number)
							assertAlmostEqual(Number(e), Number(a), message + ' at item ' + i);
						else if(e != a)
							fail("Expected "+message+" at item "+i+": " + e+ " ("+getQualifiedClassName(e)+") but was: " +a+ " ("+getQualifiedClassName(a)+")");		
					}
				} else
					fail("Expected "+message+": length " + expected.length + " [" + expected + "]\n but was: length "+actual.length+" [" + actual + "]");		
			} else if (expectedList.length > 0) {
				fail("Expected zero length, but was " + actual.length + " [" + actual + "]");		
			} else {
				//trace("zero length " + actual + " vs " + expected);
				if (actual + '' != expected + '') {
					fail("Expected "+message+": " + expected + " but was: " + actual);		
				}
			}
		}			
	}
}