package RedCorona.Tests {
	import flash.display.Sprite;
	import RedCorona.Events.DataEvent;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	import flash.utils.describeType;
	import RedCorona.Components.ScrollableRegion;
	import RedCorona.Components.Text;
	import RedCorona.Components.TextButton;
	import RedCorona.LongTasks.TaskManager;
	
	public class TestInterface extends Sprite {
		private var w:int, h:int;
		private var testClasses:Vector.<Class>;
		
		public function TestInterface( w:int, h:int, testClasses:Vector.<Class> ):void {
			this.w = w; this.h = h;
			this.testClasses = testClasses;
			if (stage) init();
			else addEventListener(Event.ADDED_TO_STAGE, init);
		}
		
		private var testRunner:TestRunner;
		
		private function init(e:Event = null):void {
			removeEventListener(Event.ADDED_TO_STAGE, init);
			// entry point
			var region:ScrollableRegion = new ScrollableRegion(w - 10, (h - 60) / 2);
			region.y = 45; region.x = 5;
			region.scrollFromBottom = true;
			addChild(region);
			
			var failures:ScrollableRegion = new ScrollableRegion(w - 10, (h - 60) / 2);
			failures.y = 45 + ((h - 40) / 2) ; failures.x = 5;
			failures.scrollFromBottom = true;
			addChild(failures);
						
			var runButton:TextButton = new TextButton("Run");
			runButton.y = 20; runButton.height = 40;
			runButton.x = 50; runButton.width = 100;
			addChild(runButton);
			runButton.addEventListener(MouseEvent.CLICK, function(e:Event):void {
				runButton.Enabled = false;
				addText(region, 'Starting test run ...', 0x606060);
				testRunner.startRun(testClasses, null);
			});
			
			testRunner = new TestRunner(new TaskManager(50, stage));
			testRunner.addEventListener(TestRunner.CLASS_START_EVENT, function(e:DataEvent) : void {
				addText(region, 'Starting class ' + e.Data, 0x606060);
			});
			testRunner.addEventListener(TestRunner.CLASS_COMPLETE_EVENT, function(e:DataEvent) : void {
				addText(region, 'Done ' + e.Data, 0x606060);
			});
			testRunner.addEventListener(TestRunner.SUITE_COMPLETE_EVENT, function(e:DataEvent) : void {
				addText(region, 'Done!', 0x606060);
				runButton.Enabled = true;
			});
			testRunner.addEventListener(TestRunner.TEST_COMPLETE_EVENT, function(e:DataEvent) : void {
				var testCase:TestCase = e.Data as TestCase;
				var mainPart:String = testCase.name + ' (' + testCase.time + ' ms) : ' + testCase.status;
				if (testCase.failed) {
					addText(region, ' FAILED: ' + mainPart , 0xA00000);
					addText(failures, testCase.hostClass + "/" + testCase.name + ': ', 0xA00000);
					addText(failures, '  '+testCase.status, 0xA00000);
					trace("TEST FAILURE: " + testCase.hostClass + "/" + testCase.name + ': ' + testCase.status);
				} else
					addText(region, ' passed: ' + mainPart, 0x00A000);
			});
			
		}
		
		
		private function addText(region:ScrollableRegion, text:String, col:uint) : void {
			var textField:TextField = Text.MakeTextField(600, text, col, 1, 'left');
			textField.y = region.Scrollbar.Max;
			region.addChild(textField);
		}		
	}
}