package RedCorona.Tests {;
	import flash.events.EventDispatcher;
	import flash.utils.getTimer;
	import RedCorona.LongTasks.LoopTask;
	import RedCorona.LongTasks.MarkerTask;
	import RedCorona.LongTasks.TaskManager;
	import RedCorona.Events.DataEvent;
	
	import flash.utils.describeType;
	
	/**
	* ...
	* @author 
	*/
	public class TestRunner extends EventDispatcher {
		public static const TEST_COMPLETE_EVENT:String = "testComplete",
			CLASS_COMPLETE_EVENT:String = "classComplete",
			TEST_START_EVENT:String = 'testStart',
			CLASS_START_EVENT:String = 'classStart',
			SUITE_COMPLETE_EVENT:String = 'suiteComplete';
		
		private var taskManager:TaskManager;
			
		public function TestRunner(taskManager:TaskManager) {
			this.taskManager = taskManager;
			taskManager.addEventListener(TaskManager.TASK_COMPLETE_EVENT, function(e: DataEvent) : void {
				var marker:MarkerTask = e.Data as MarkerTask;
				if (marker != null) {
					dispatchEvent(marker.source);
				}
			});
		}
		
		public function startRun(classes:Vector.<Class>, identifier:*) : void {
			for each(var c:Class in classes) {
				taskManager.add(new MarkerTask(new DataEvent(CLASS_START_EVENT, c)));
				var xml:XML = describeType(c);
				var testMethods:Vector.<*> = new Vector.<*>();
				for each(var method:XML in xml.factory.method) {
					var metadata:XMLList = method.metadata.(@name == "Test");
					if(metadata.length() > 0){
						testMethods.push({xml:method, metadata:metadata, host:c});
					}
				}
				
				
				taskManager.add(new LoopTask(Vector.<*> (testMethods), function(method:*) : void {
					var methodXML:XML = method.xml;
					for each(var metaitem:XML in method.metadata) {
						var params:Array = getMetadataParams(metaitem);
						var testCase:TestCase = TestCase.startTest(methodXML.@name + getMetadataText(params));
						testCase.hostClass = method.host;
						var time:int = getTimer();
						dispatchEvent(new DataEvent(TEST_START_EVENT, testCase ));
						try {
							var target:* = new method.host();
							target[methodXML.@name].apply(target, params);
						} catch (e: * ) {
							testCase.fail("Threw " + e);
						}
						testCase.time = getTimer() - time;
						dispatchEvent(new DataEvent(TEST_COMPLETE_EVENT, testCase ));
					}
				}, this));
				taskManager.add(new MarkerTask(new DataEvent(CLASS_COMPLETE_EVENT, c)));
			}
			taskManager.add(new MarkerTask(new DataEvent(SUITE_COMPLETE_EVENT, identifier)));
		}
		
		private function getMetadataParams(metadata:XML) : Array {
			var r:Array = [];
			var next:int = 1;

			for each(var arg:XML in metadata.arg) {
				r.push(arg.@value.toString());
				//trace("param " + arg.@value);
			}
			return r;
		}
		
		private function getMetadataText(params:Array) : String {
			if (0 == params.length) return '';
			else
				return ' with args [' + params.join(', ') + ']';			
		}
	}

}